--=============================================================================--
--===========================================================================--
library ieee;
   use ieee.std_logic_1164.all;
   use IEEE.STD_LOGIC_ARITH.ALL;
   use IEEE.STD_LOGIC_UNSIGNED.ALL;
   use ieee.numeric_std.all;

entity System09_Terasic_DE1 is
  port(
	-- clocks
	clock_24    : in    std_logic_vector(1 downto 0);
	clock_27    : in    std_logic; 
    clock_50    : in    std_logic;  -- System Clock input
    ext_clock   : in    std_logic;
    
    -- push button keys
	key         : in    std_logic_vector(3 downto 0);  -- RST, NMI

	-- LEDS & Switches
	ledg        : out std_logic_vector(7 downto 0);
	ledr        : out std_logic_vector(9 downto 0);
	sw          : in  std_logic_vector(9 downto 0);
	
	--Analog
	gpio_0      : inout   std_logic_vector(35 downto 0);
	--gpio_0	: inout std_logic_vector ( 7 downto 0);  --VIA IO
	--gpio_0  : out std_logic_vector (16 downto 8);	 --VIA Addr,CLK,CS etc
	--gpio_0  : inout std_logic_vector (24 downto 17); --VIA PortA
	--gpio_0  : in std_logic_vector (32 downto 25);	 --VIA PortB
	
	gpio_1      : out   std_logic_vector(35 downto 0);
	
	--Hex
	hex0	    : out std_logic_vector (0 to 6);
	hex1	    : out std_logic_vector (0 to 6);
	hex2	    : out std_logic_vector (0 to 6);
	hex3	    : out std_logic_vector (0 to 6)
	
	);
end System09_Terasic_DE1;

-------------------------------------------------------------------------------
-- Architecture for System09
-------------------------------------------------------------------------------
architecture my_computer of System09_Terasic_DE1 is
  -----------------------------------------------------------------------------
  -- constants
  -----------------------------------------------------------------------------
  constant SYS_CLK_FREQ  : integer := 50000000;  -- FPGA System Clock
  constant VGA_CLK_FREQ  : integer := 25000000;  -- VGA Pixel Clock
  constant CPU_CLK_FREQ  : integer := 12000000;  -- CPU Clock

  type hold_state_type is ( hold_release_state, hold_request_state );

  -----------------------------------------------------------------------------
  -- Signals
  -----------------------------------------------------------------------------

  signal sys_clk       	: std_logic; 
--signal vga_clk       	: std_logic;
  signal pll_locked    	: std_logic;
  signal resetsr 		: std_logic_vector(1 downto 0);
  signal ares			: std_logic;
  signal cnt1m5			: integer range 0 to 7; 
  signal clk1m5			: std_logic;
  signal cnt6m			: integer range 0 to 1; 
  signal clk6m			: std_logic;
  signal cnt750k		: integer range 0 to 7; 
--signal clk750k		: std_logic;

  
  -- CPU Interface signals
  signal cpu_clk       : std_logic;
  signal cpu_rst       : std_logic;
  signal cpu_rw        : std_logic;
  signal cpu_nrw	   : std_logic;
  signal cpu_vma       : std_logic;
  signal cpu_nvma	   : std_logic;
--signal cpu_halt      : std_logic;
--signal cpu_hold      : std_logic;
  signal cpu_firq      : std_logic;
--signal cpu_irq       : std_logic;
--signal cpu_nmi       : std_logic;
  signal cpu_addr      : std_logic_vector(15 downto 0);
  signal cpu_data_in   : std_logic_vector(7 downto 0);
  signal cpu_data_out  : std_logic_vector(7 downto 0);
  signal cpu_pc		   : std_logic_vector(15 downto 0);

  -- BOOT ROM
  signal rom_cs        : Std_logic;
  signal rom_data_out  : Std_Logic_Vector(7 downto 0);

  -- IO / VIA
  signal io_cs       : std_logic;
  signal io_data_out : std_logic_vector(7 downto 0);
  signal io_data_oe  : std_logic;
  signal via_wen	 : std_logic;
  signal via_ren     : std_logic;
  signal via_pa_out  : std_logic_vector(7 downto 0);
  signal via_pa_oe	 : std_logic_vector(7 downto 0);
  signal via_pa_in   : std_logic_vector(7 downto 0);
  signal via_pb_out  : std_logic_vector(7 downto 0);
  signal via_pb_oe	 : std_logic_vector(7 downto 0);
  signal via_pb_in	 : std_logic_vector(7 downto 0);
--signal via_irq_out : std_logic; -- ?????????????
  signal via_ca1_in  : std_logic; -- to IO7 AY8912
  signal via_ca2_out : std_logic; -- alg n_zero
--signal via_ca2_oe  : std_logic; -- alg n_zero
  signal via_ca2_in  : std_logic; -- alg n_zero
  signal via_cb1_out : std_logic; -- nc
--signal via_cb1_oe  : std_logic; -- nc
  signal via_cb1_in  : std_logic; -- nc
  signal via_cb2_out : std_logic; -- alg n_blank
--signal via_cb2_oe  : std_logic; -- alg n_blank
  signal via_cb2_in  : std_logic; -- alg n_blank
  
-- echter VIA
signal via_data_out : std_logic_vector(7 downto 0);
signal cpu_data_out_int : std_logic_vector(7 downto 0);
signal via_wr : std_logic;
signal via_rd : std_logic;
  
  -- Sound / AY
  signal ay_bc1		 : std_logic; -- to AY bc1
  signal ay_bdir     : std_logic; -- to AY bdir
  signal ay_pa_out	 : std_logic_vector(7 downto 0);
  signal ay_data_out : std_logic_vector(7 downto 0);
  signal ay_data_in  : std_logic_vector(7 downto 0);
--signal ay_data_oe  : std_logic;
  signal analog_sound    : std_logic_vector(7 downto 0);
  
  -- CART
  signal cart_cs       : std_logic;
  signal cart_data_out : std_logic_vector(7 downto 0);

  -- RAM
  signal sram_cs       : std_logic;
  signal sram_data_out : std_logic_vector(7 downto 0);
  signal sram_ce       : std_logic;
  signal sram_ub       : std_logic;
  signal sram_lb       : std_logic;
  signal sram_we       : std_logic;
  signal sram_oe       : std_logic;
  
  --Analog
  signal analog_vec    : std_logic_vector(7 downto 0);
  signal analog_x      : std_logic_vector(7 downto 0);
  signal analog_y      : std_logic_vector(7 downto 0);
  signal analog_y_temp : std_logic_vector(7 downto 0);
  signal analog_x_int  : std_logic_vector(7 downto 0);
  signal analog_y_int  : std_logic_vector(7 downto 0);
  signal analog_z      : std_logic_vector(7 downto 0);
  signal analog_z_temp : std_logic_vector(7 downto 0);
  signal analog_offs   : std_logic_vector(7 downto 0);
  signal ramp_int	   : std_logic;
  signal zero_int	   : std_logic;
--signal cs_x          : std_logic;
--signal cs_y          : std_logic;
--signal cs_x_int      : std_logic;
--signal cs_y_int      : std_logic;
--signal cs_z          : std_logic;


-----------------------------------------------------------------
--
-- CPU09 CPU core
--
-----------------------------------------------------------------

--component cpu09
--  port (    
--	 clk      :	in  std_logic;
--     rst      : in  std_logic;
--     vma      :	out std_logic;
--     addr     : out std_logic_vector(15 downto 0);
--     --address:	 out std_logic_vector(15 downto 0);
--     rw       :	out std_logic;
--     data_in  : in	 std_logic_vector(7 downto 0);
--	 data_out : out std_logic_vector(7 downto 0);
--	 halt     : in  std_logic;
--	 hold     : in  std_logic;
--	 irq      : in  std_logic;
--	 nmi      : in  std_logic;
--	 firq     : in  std_logic
--	 --dpc      : out std_logic_vector (15 downto 0)
--  );
--end component;


component cpu6809
    port
    (
       -- system
       resetn   :  in std_logic;
       clk      :  in std_logic;
       clken    :  in std_logic;
       -- interruptions
       nnmi     :  in std_logic;
       nirq     :  in std_logic;
       nfiq     :  in std_logic;
       -- memory
       busreq   : out std_logic;
       write    : out std_logic;
       addr     : out std_logic_vector(15 downto 0);
       rdata    :  in std_logic_vector( 7 downto 0);
       wdata    : out std_logic_vector( 7 downto 0)
    );
  end component;


----------------------------------------
--
-- 8KByte Block RAM as Main ROM
--
----------------------------------------
component mon_rom
  Port (
    clk      : in  std_logic;
    rst      : in  std_logic;
    cs       : in  std_logic;
    rw       : in  std_logic;
    addr     : in  std_logic_vector (12 downto 0);
    data_in  : in  std_logic_vector (7 downto 0);
    data_out : out std_logic_vector (7 downto 0)
    );
end component;

component cart_rom
  Port (
    clk      : in  std_logic;
    rst      : in  std_logic;
    cs       : in  std_logic;
    rw       : in  std_logic;
    addr     : in  std_logic_vector (12 downto 0);
    data_in  : in  std_logic_vector (7 downto 0);
    data_out : out std_logic_vector (7 downto 0)
    );
end component;

component ram_2k
 Port (
    clk      : in  std_logic;
    rst      : in  std_logic;
    cs       : in  std_logic;
    rw       : in  std_logic;
    addr     : in  std_logic_vector (9 downto 0);
    data_in  : in  std_logic_vector (7 downto 0);
    data_out : out std_logic_vector (7 downto 0)
    );
end component;

component pll IS
	PORT
	(
		areset		: IN STD_LOGIC  := '0';
		inclk0		: IN STD_LOGIC  := '0';
		c0			: OUT STD_LOGIC ;
		c1			: OUT STD_LOGIC ;
		c2			: OUT STD_LOGIC ;
		locked		: OUT STD_LOGIC 
	);
end component;


component via6522 is
port (
    clock       : in  std_logic;
    clock_en    : in  std_logic; -- for counters and stuff
    reset       : in  std_logic;
    
    addr        : in  std_logic_vector(3 downto 0);
    wen         : in  std_logic;
    ren         : in  std_logic;
    data_in     : in  std_logic_vector(7 downto 0);
    data_out    : out std_logic_vector(7 downto 0);

    -- pio --
    port_a_o    : out std_logic_vector(7 downto 0);
    port_a_t    : out std_logic_vector(7 downto 0);
    port_a_i    : in  std_logic_vector(7 downto 0);
    
    port_b_o    : out std_logic_vector(7 downto 0);
    port_b_t    : out std_logic_vector(7 downto 0);
    port_b_i    : in  std_logic_vector(7 downto 0);

    -- handshake pins
    ca1_i       : in  std_logic;

    ca2_o       : out std_logic;
    ca2_i       : in  std_logic;
    ca2_t       : out std_logic;
    
    cb1_o       : out std_logic;
    cb1_i       : in  std_logic;
    cb1_t       : out std_logic;
    
    cb2_o       : out std_logic;
    cb2_i       : in  std_logic;
    cb2_t       : out std_logic;

    irq         : out std_logic
      );
end component;


--component M6522 is
--    port (
--      RS              : in    std_logic_vector(3 downto 0);
--      DATA_IN         : in    std_logic_vector(7 downto 0);
--      DATA_OUT        : out   std_logic_vector(7 downto 0);
--      DATA_OUT_OE_L   : out   std_logic;
--
--      RW_L            : in    std_logic;
--      CS1             : in    std_logic;
--      CS2_L           : in    std_logic;
--
--      IRQ_L           : out   std_logic; -- note, not open drain
--
--      CA1_IN          : in    std_logic;
--      CA2_IN          : in    std_logic;
--      CA2_OUT         : out   std_logic;
--      CA2_OUT_OE_L    : out   std_logic;
--
--      PA_IN           : in    std_logic_vector(7 downto 0);
--      PA_OUT          : out   std_logic_vector(7 downto 0);
--      PA_OUT_OE_L     : out   std_logic_vector(7 downto 0);
--
--      -- port b
--      CB1_IN          : in    std_logic;
--      CB1_OUT         : out   std_logic;
--      CB1_OUT_OE_L    : out   std_logic;
--
--      CB2_IN          : in    std_logic;
--      CB2_OUT         : out   std_logic;
--      CB2_OUT_OE_L    : out   std_logic;
--
--      PB_IN           : in    std_logic_vector(7 downto 0);
--      PB_OUT          : out   std_logic_vector(7 downto 0);
--      PB_OUT_OE_L     : out   std_logic_vector(7 downto 0);
--
--      RESET_L         : in    std_logic;
--      P2_H            : in    std_logic; -- high for phase 2 clock  ____----__
--      CLK_4           : in    std_logic  -- 4x system clock (4HZ)   _-_-_-_-_-
--      );
--end component;

component PWM is
      Port ( 
            clk      : in  std_logic;
            pwmvalue : in  std_logic_vector (7 downto 0);
            pwmout   : out std_logic
            );
end component;

component SPRITE_PWM is
      Port ( 
           clk      	: in  STD_LOGIC;
           pwmvalue 	: in  STD_LOGIC_VECTOR (7 downto 0);
           pwm_stb  	: in std_logic;
           pwmout_h   	: out STD_LOGIC;
           pwmout_l   	: out STD_LOGIC
            );
end component;

component signed_integ is
      Port ( 
			  clk_i              :  in  std_logic;
			  rst_i              :  in  std_logic;
			  ramp_i			 :  in  std_logic;
			  zero_i			 :  in  std_logic;
			  load_offset_i		 :  in  std_logic;
			  data_i          	 :  in  std_logic_vector(7 downto 0);
			  offset_i			 :  in  std_logic_vector(7 downto 0);
			  data_o        	 :  out std_logic_vector(7 downto 0);
			  data_o_stb		 :  out std_logic
            );
end component;

component hexdisplay is
    Port( code  	:  in STD_LOGIC_VECTOR (3 downto 0);
          seg7   	:  out STD_LOGIC_VECTOR (0 to 6)
         );
end component;

component YM2149 is
	Port (
		  -- data bus
		  I_DA                : in  std_logic_vector(7 downto 0);
		  O_DA                : out std_logic_vector(7 downto 0);
		  O_DA_OE_L           : out std_logic;
		  -- control
		  I_A9_L              : in  std_logic;
		  I_A8                : in  std_logic;
		  I_BDIR              : in  std_logic;
		  I_BC2               : in  std_logic;
		  I_BC1               : in  std_logic;
		  I_SEL_L             : in  std_logic;

		  O_AUDIO             : out std_logic_vector(7 downto 0);
		  -- port a
		  I_IOA               : in  std_logic_vector(7 downto 0);
		  O_IOA               : out std_logic_vector(7 downto 0);
		  O_IOA_OE_L          : out std_logic;
		  -- port b
		  I_IOB               : in  std_logic_vector(7 downto 0);
		  O_IOB               : out std_logic_vector(7 downto 0);
		  O_IOB_OE_L          : out std_logic;
		  --
		  ENA                 : in  std_logic; -- clock enable
		  RESET_L             : in  std_logic; -- E clock , 1.5MHz
		  CLK                 : in  std_logic  -- 6 Mhz
		);
end component;


begin
  -----------------------------------------------------------------------------
  -- Instantiation of internal components
  -----------------------------------------------------------------------------

--my_cpu : cpu09  port map (    
--	 clk	   => clk1m5,
--	 rst       => cpu_rst,
--     vma       => cpu_vma,
--     addr      => cpu_addr(15 downto 0),
--     --address   => cpu_addr(15 downto 0),
--     rw	       => cpu_rw,
--     data_in   => cpu_data_in,
--	 data_out  => cpu_data_out,
--	 halt      => '0',
--	 hold      => '0',
--	 irq       => '0', 
--	 nmi       => '0', 
--	 firq      => '0'
--	 --dpc	   => cpu_pc
--    );
    
    
cpu_0:cpu6809
  port map
  (
    -- system                                       
    resetn   => not cpu_rst,
    clk      => clk6m,
    clken    => clk1m5,
    -- interruptions                                
    nnmi     => '1',
    nirq     => '1',        
    nfiq     => '1',
    -- memory                                       
    busreq   => cpu_vma,
    write    => cpu_nrw,
    addr     => cpu_addr(15 downto 0),
    rdata    => cpu_data_in,
    wdata    => cpu_data_out
  );

cpu_rw <= not cpu_nrw;
--cpu_vma <= '1';
--cpu_vma <= cpu_vma;

my_rom : mon_rom port map (
    clk       => clk6m,
    rst       => cpu_rst,
	cs        => rom_cs,
	rw        => '1',
    addr      => cpu_addr(12 downto 0),
    data_in   => cpu_data_out,
    data_out  => rom_data_out
    );
    
my_cart : cart_rom port map (
    clk       => clk6m,
    rst       => cpu_rst,
	cs        => cart_cs,
	rw        => '1',
    addr      => cpu_addr(12 downto 0),
    data_in   => cpu_data_out,
    data_out  => cart_data_out
    );
    
my_ram : ram_2k port map (
	clk       => clk6m,
    rst       => cpu_rst,
	cs        => sram_cs,
	rw        => not cpu_rw ,
    addr      => cpu_addr(9 downto 0),
    data_in   => cpu_data_out,
    data_out  => sram_data_out
	);

my_via : via6522 port map (
    
		clock       => cpu_clk,
        clock_en    => clk1m5,
        reset       => cpu_rst,
                                
        addr        => cpu_addr(3 downto 0),
        wen         => not cpu_rw and io_cs,
        ren         => cpu_rw and io_cs,
        data_in     => cpu_data_out,
        data_out    => io_data_out,
                                
        -- pio --   
        port_a_o    => via_pa_out,
        port_a_t    => via_pa_oe,
        port_a_i    => via_pa_in,
                                
        port_b_o    => via_pb_out,
        port_b_t    => via_pb_oe,
        port_b_i    => via_pb_in,
    
        -- handshake pins
        ca1_i       => ay_pa_out(7),--via_ca1_in,
                            
        ca2_o       => via_ca2_out,
        ca2_i       => via_ca2_in,
--      ca2_t       => via_ca2_oe,
                            
        cb1_o       => via_cb1_out,
        cb1_i       => via_cb1_in,
--      cb1_t       => via_cb1_oe,
                            
        cb2_o       => via_cb2_out,
        cb2_i       => via_cb2_in
--      cb2_t       => via_cb2_oe,
                            
--      irq         => via_irq_out
    );

--my_via : M6522 port map (
--    RS              => cpu_addr(3 downto 0),
--    DATA_IN         => cpu_data_out,
--    DATA_OUT        => io_data_out,
--    DATA_OUT_OE_L   => open,
--
--    RW_L            => cpu_rw,
--    CS1             => cpu_addr(12),
--    CS2_L           => not io_cs,
--
--    IRQ_L           => open, --via1_nmi_l, -- note, not open drain
--
--	-- port b
--	  
--    CA1_IN          => '1',--ay_pa_out(7),
--    CA2_IN          => via_ca2_out,
--    CA2_OUT         => via_ca2_out,
--   -- CA2_OUT_OE_L    => open,
--
--    PA_IN           => via_pa_in, 
--    PA_OUT          => via_pa_out, --ledr(7 downto 0)
--    --PA_OUT_OE_L     => via_pa_oe,
--
--      -- port b
--    CB1_IN          => via_cb1_out,
--    CB1_OUT         => via_cb1_out,
--    --CB1_OUT_OE_L    => open,
--
--    CB2_IN          => via_cb2_out,
--    CB2_OUT         => via_cb2_out,
--    --CB2_OUT_OE_L    => open,
--
--    PB_IN           => via_pb_in,
--    PB_OUT          => via_pb_out,
--    --PB_OUT_OE_L     => via_pb_oe,
--
--    RESET_L         => not cpu_rst,
--    P2_H            => not clk1m5,
--    CLK_4           => clk6m
--    );
--    
my_ay : YM2149
	port map (
	-- data bus
    I_DA                => ay_data_in,
    O_DA                => ay_data_out,
 --   O_DA_OE_L           => ay_data_oe,
    -- control
    I_A9_L              => '0',
    I_A8                => '1',
    I_BDIR              => ay_bdir,
    I_BC2               => '1',
    I_BC1               => ay_bc1,
    I_SEL_L             => '1',

    O_AUDIO             => analog_sound,
    -- port a
    I_IOA               => "11111111",	--joysticks
    O_IOA               => ay_pa_out,
    O_IOA_OE_L          => open,		--ioa_oe_l,
    -- port b
    I_IOB               => "00000000", 	--portb nc at AY38912
    O_IOB               => open, 		--portb nc at AY38912
    O_IOB_OE_L          => open, 		--portb nc at AY38912

    ENA                 => '1',
    RESET_L             => not cpu_rst,
    CLK                 => clk1m5
    );


x_int : signed_integ
	port map (
			clk_i      		=> clk6m,
			rst_i      		=> cpu_rst,
			ramp_i	   		=> not ramp_int,
			zero_i	   		=> not zero_int,
			load_offset_i	=> '1',
			data_i			=> analog_x(7 downto 0),
			offset_i		=> analog_offs(7 downto 0),
			data_o			=> analog_x_int(7 downto 0)
--			data_o_stb		=> '1'
			);
			
y_int : signed_integ
	port map (
			clk_i      		=> clk6m,
			rst_i      		=> cpu_rst,
			ramp_i	   		=> not ramp_int,
			zero_i	   		=> not zero_int,
			load_offset_i	=> '1',
			data_i			=> analog_y(7 downto 0),
			offset_i		=> analog_offs(7 downto 0),
			data_o			=> analog_y_int(7 downto 0)
--			data_o_stb		=> '1'
			);
        
sound_pwm : PWM
   port map( CLK => clk6m,
             pwmvalue => analog_sound,
			 pwmout => gpio_1(0)
           );
           
x_pwm : SPRITE_PWM
   port map( CLK => sys_clk,
             pwmvalue => ( not analog_x_int(7) & ( analog_x_int(6 downto 0) ) ),-- + 1,--analog_x_int(7 downto 0),
             pwm_stb  => clk1m5,
			 pwmout_h => gpio_1(1),
			 pwmout_l => gpio_1(2)
           );

y_pwm : SPRITE_PWM
   port map( CLK => sys_clk,
             pwmvalue => ( not analog_y_int(7) & ( analog_y_int(6 downto 0) ) ),-- + 1,--analog_y_int(7 downto 0),
             pwm_stb  => clk1m5,
			 pwmout_h => gpio_1(3),
			 pwmout_l => gpio_1(4)
           );

--z_pwm : SPRITE_PWM
--   port map( CLK => sys_clk,
--             pwmvalue => ( not analog_offs(7) & ( analog_offs(6 downto 0) ) ),
--             pwm_stb  => '1'                                                             ,
--			 pwmout_h => gpio_1(5),
--			 pwmout_l => gpio_1(6)
--           );

analog_zeroref_pwm : PWM   
    port map( CLK => clk6m,
             pwmvalue => ( not analog_offs(7) & ( analog_offs(6 downto 0) ) ),
			 pwmout => gpio_1(5)
           );

chex0 : hexdisplay
	port map(
			code => cpu_pc( 3 downto 0),
			seg7 => hex0(0 to 6)
			);
chex1 : hexdisplay
	port map(
			code => cpu_pc( 7 downto 4),
			seg7 => hex1(0 to 6)
			);
chex2 : hexdisplay
	port map(
			code => cpu_pc( 11 downto 8),
			seg7 => hex2(0 to 6)
			);
chex3 : hexdisplay
	port map(
			code => cpu_pc( 15 downto 12),
			seg7 => hex3(0 to 6)
			);

----------------------------------------
--
-- Phase Locked Loop Clock divider
--

my_pll : pll
	PORT MAP
	(
		areset		=> '0',
		inclk0		=> clock_50,
		c0			=> sys_clk,
		c1			=> cpu_clk,
--		c2			=> vga_clk,
		locked		=> pll_locked 
	);
	 
----------------------------------------------------------------------
--
-- Process to decode memory map
--
----------------------------------------------------------------------

mem_decode: process(cpu_rst,cpu_data_out,key,resetsr,cpu_addr,cart_data_out,sw,
					cpu_vma,sram_data_out,io_data_out,rom_data_out)
begin
	if rising_edge (cpu_clk) then

		  cpu_rst    <= resetsr(1); -- CPU reset is active high	  
		  ares		 <= not key(0);		
		
		if cpu_rst = '1' then
		  cpu_data_in <= (others=>'0');
		  rom_cs      <= '0';
		  io_cs       <= '0';
		  sram_cs     <= '0';
		  cart_cs	  <= '0';      
		else
		 

	-- Memory Decoder

		if cpu_addr( 15 ) = '0' then 				--CART 0x0000-0x7FFF
			cart_cs     <= cpu_vma;
			--cart_cs     <= '1';
			if sw(1) = '1' then
				cpu_data_in <= "00000000";
				cart_cs <= '0';
			else
				cart_cs     <= cpu_vma;
				cpu_data_in <= cart_data_out;
			end if;
		else
			--cpu_data_in <= cpu_data_in;
			cart_cs <= '0';
		end if;  

		if cpu_addr( 15 downto 11 ) = "11001" then 	--RAM 0xC800-0xCFFF
		--if cpu_addr( 15 downto 12 ) = "1100" then 	--RAM 0xC800-0xCFFF
			cpu_data_in <= sram_data_out;
			sram_cs     <= cpu_vma;
			--sram_cs     <= '1';
		else
			--cpu_data_in <= cpu_data_in;
			sram_cs <= '0';
		end if;
		
		if cpu_addr( 15 downto 12 ) = "1101" then 	--IO 0xD000-0xDFFF	    
			cpu_data_in <= io_data_out;	--VHDL VIA
			--cpu_data_in <= via_data_out;	--Real VIA
			io_cs     <= cpu_vma;
			--io_cs     <= '1';			
		else
			--cpu_data_in <= cpu_data_in;
			io_cs <= '0';	
		end if;

		if cpu_addr( 15 downto 13 ) = "111" then 	--ROM 0xE000-0xFFFF
			cpu_data_in <= rom_data_out;
			rom_cs      <= cpu_vma;
			--rom_cs      <= '1';
		else
			--cpu_data_in <= cpu_data_in;
			rom_cs <= '0';
		end if;
	end if;		
  end if;      
end process;


-- LEDS output register

led_output : process( via_cb1_out , sram_cs , rom_cs , cart_cs , io_cs )
begin
	ledr(9) <= cart_cs; 
	ledr(8) <= io_cs; 
	ledr(7) <= sram_cs;
	ledr(6) <= rom_cs;
	ledr(5) <= via_cb1_out;
end process;

led_output2 : process( clk1m5, cpu_rst, sram_cs, cpu_rw, cpu_addr, cpu_data_out)
begin
	if clk1m5'event and clk1m5='0' then
		if cpu_rst = '1' then
			ledg <= (others=>'0');
		else
			--ledg(7 downto 0) <= cpu_addr(15 downto 8);
			ledg(7 downto 0) <= via_data_out;
		end if;
	end if;
end process;


update_alg : process( clk6m, clk1m5 ,cpu_rst,via_pb_out,via_cb1_out,via_pb_oe,via_ca2_out,
					via_cb2_out,analog_vec,analog_y_temp,via_pa_oe,ay_data_out,
					analog_x,via_pa_out)

-- Resolution: 330  410

begin

gpio_0(8)				<= clk1m5;  --clock		
if rising_edge (cpu_clk) then
	
----START REAL VIA	
--					
--		if cpu_rst = '1' then		
--			ay_bc1		   			<= '0';
--			ay_bdir        			<= '0';
--			gpio_0(16)				<= '0'; --reset
--	
--		else			
--			gpio_0(16)				<= '1'; 	--reset			
--			gpio_0(15 downto 12)    <= cpu_addr(3 downto 0); --addr
--			gpio_0(11)				<= cpu_rw;
--			gpio_0(9)				<= cpu_addr(12); --cs1
--			gpio_0(10)				<= not io_cs;   --#cs2
--			
--			ay_bc1		<= gpio_0(30);	--AY cs
--			ay_bdir     <= gpio_0(31);	--AY direction
--			ramp_int	<= gpio_0(32);	--ramp
--			zero_int    <= gpio_0(25);	--zero integrators
--			
--			via_wr <= not cpu_rw and io_cs and cpu_addr(12);
--			via_rd <= cpu_rw and io_cs and cpu_addr(12);
--			
--			if io_cs = '1' then
--			   cpu_data_out_int <= cpu_data_out;
--			end if;
--				
--			if via_wr = '1' then				
--				gpio_0(7 downto 0)  <= cpu_data_in;
--			end if;
--			
--			if via_rd = '1' then
--				gpio_0(7 downto 0)  <= (others=>'Z');
--				via_data_out		<= gpio_0(7 downto 0);
--			end if;	
--			
--				--gpio_0(7 downto 0)  <= (others=>'Z');
--				--via_data_out		<= gpio_0(7 downto 0);
--					
--			
--			if (ay_bdir = '1') and (ay_bc1 = '1') then
--				--gpio_0(24 downto 17)  <= (others=>'Z');
--				gpio_0(24 downto 17) <= ay_data_out;
--			end if;
--				
--				ay_data_in			  <= gpio_0(24 downto 17);
--				analog_vec			  <= gpio_0(24 downto 17);
--				gpio_0(24 downto 17)  <= (others=>'Z');
--
--			
--			--	ay_data_in			  <= gpio_0(24 downto 17);
--			--	analog_vec			  <= gpio_0(24 downto 17);
--			
--
--			if gpio_0(26) = '0' then		--blanking
--				gpio_1(6) <= '1';
--				else
--				gpio_1(6) <= '0';
--				analog_z <= "00000000";
--			end if;			
--			
--			
--			if gpio_0(27) = '0'  then 	-- mux enable
--			
--				--demux y channel 			
--				if 	(gpio_0(28) = '0') and (gpio_0(29) = '0') then
--					analog_y 	<= analog_vec;
--				end if;			
--				
--				--demux z channel
--				if 	(gpio_0(28) = '0') and (gpio_0(29) = '1') then
--					analog_z 	<= analog_vec;
--				end if;
--				
--				--demux offset channel
--				if 	(gpio_0(28) = '1') and (gpio_0(29) = '0') then
--					analog_offs <= analog_vec;
--				end if;
--				
--			end if;
--			
--			analog_x 	<= analog_vec;	--analog_vec;	
--				
--		end if;
----END REAL VIA	

--START VHDL VIA		
		if cpu_rst = '1' then		
			ay_bc1		   			<= '0';
			ay_bdir        			<= '0';
			via_pa_in				<= "00000000";
			via_pb_in(5)			<= '1';
			via_pb_in(6)			<= '1';		
		else			
			via_pb_in	<= via_pb_out;		--portb via	
			via_cb1_in	<= via_cb1_out;		--via cb1
			ay_bc1		<= via_pb_out(3);	--AY cs
			ay_bdir     <= via_pb_out(4);	--AY direction
			via_ca2_in	<= via_ca2_out; 	--nZero			
			via_cb2_in	<= via_cb2_out;		--nBlank, Gruen			
			analog_x 	<= via_pa_out;		--analog_vec;

			zero_int	<= via_ca2_out;
			ramp_int	<= via_pb_out(7);

			if via_cb2_out = '0' then		--blanking
				gpio_1(6) <= '1';
				else
				gpio_1(6) <= '0';
				analog_z <= "00000000";
			end if;
			
			if via_pb_out(0) = '0'  then 	-- mux enable
			
				--demux y channel 			
				if 	(via_pb_out(1) = '0') and (via_pb_out(2) = '0') then
					analog_y 	<= analog_vec;
				end if;			
				
				--demux z channel
				if 	(via_pb_out(1) = '0') and (via_pb_out(2) = '1') then
					analog_z 	<= analog_vec;
				end if;
				
				--demux offset channel
				if 	(via_pb_out(1) = '1') and (via_pb_out(2) = '0') then
					analog_offs <= analog_vec;
				end if;
				
			end if;
			
			if via_pa_oe = "00000000" then		--Read VIA PortA	
				
					if via_pb_out(3) = '1' and via_pb_out(4) = '0' then
						via_pa_in		<= ay_data_out;
					else
						via_pa_in	    <=  analog_vec;
					end if;						
			else								--Write VIA PortA				
					via_pa_in		<= via_pa_out;	
					ay_data_in		<= via_pa_out;
					analog_vec		<= via_pa_out;				
			end if;			
		end if;
--END VHDL VIA
	end if;	
end process;

reset : process begin 
   wait until rising_edge(clk1m5) and pll_locked = '1';
   resetsr <= resetsr(0) & ares;
end process;

vex_clk : process (cpu_clk,clk6m,clk1m5)
begin
	if rising_edge(cpu_clk) then
		if cnt6m = 2-1 then
		   clk6m  <= '1';
		   cnt6m  <= 0;
		else
		   clk6m  <= '0';
		   cnt6m  <= cnt6m +1;
		end if;
	 end if; 
   
	if rising_edge(clk6m) then       
		if cnt1m5  = 4-1 then
		   clk1m5  <= '1';
		   cnt1m5  <= 0;
		else
		   clk1m5  <= '0';
		   cnt1m5  <= cnt1m5 +1;
		end if;
	 end if;   
end process;


end my_computer; --===================== End of architecture =======================--

