library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

use work.pkg_vectrex.all;
use work.pkg_d2a.all;

entity vectrex_xlink is
    Port ( O_PWM_XH : out  STD_LOGIC;
           O_PWM_XL : out  STD_LOGIC;
           O_PWM_YH : out  STD_LOGIC;
           O_PWM_YL : out  STD_LOGIC;
           O_VIDEO_Z : out  STD_LOGIC_VECTOR(3 downto 0);
           O_RDIS : out  STD_LOGIC;
           O_GDIS : out  STD_LOGIC;
           O_BDIS : out  STD_LOGIC;
			  O_DBG: out STD_LOGIC_VECTOR(15 downto 0);
			  I_PSXDATA				  : in    std_logic;
			  O_PSXCMD				  : out   std_logic;
			  O_PSXATT				  : out   std_logic;
           O_PSXCLK				  : out   std_logic;
	        I_PSXACK				  : in    std_logic;
           I_CLK : in  STD_LOGIC;
           I_RESET : in  STD_LOGIC;
           O_AUDIO_L : out  STD_LOGIC;
           O_AUDIO_R : out  STD_LOGIC);
end vectrex_xlink;


architecture Behavioral of vectrex_xlink is
signal player1_buttons:		STD_LOGIC_VECTOR(3 downto 0);
signal player2_buttons:		STD_LOGIC_VECTOR(3 downto 0);
signal player1_x:		STD_LOGIC_VECTOR(7 downto 0);
signal player1_y:		STD_LOGIC_VECTOR(7 downto 0);
signal player2_x:		STD_LOGIC_VECTOR(7 downto 0);
signal player2_y:		STD_LOGIC_VECTOR(7 downto 0);
signal psxbuttons:	STD_LOGIC_VECTOR(15 downto 0);
signal clk_ibuf:		STD_LOGIC;
signal clk_dlla_op0:	STD_LOGIC;
signal clk_dlla_div:	STD_LOGIC;
signal clk_dlla_2x:	STD_LOGIC;
signal reset_h:		STD_LOGIC;
signal reset_l:		STD_LOGIC;
signal clk_104:		STD_LOGIC;
signal clk_12:			STD_LOGIC;
signal clke_psx:		STD_LOGIC;
signal clkc_psx:		STD_LOGIC_VECTOR(3 downto 0);
signal clk_pwm:		STD_LOGIC;
signal xval:			STD_LOGIC_VECTOR(9 downto 0);
signal yval:			STD_LOGIC_VECTOR(9 downto 0);
signal xval_filt:			STD_LOGIC_VECTOR(9 downto 0);
signal yval_filt:			STD_LOGIC_VECTOR(9 downto 0);
signal xval_pwm:			STD_LOGIC_VECTOR(9 downto 0);
signal yval_pwm:			STD_LOGIC_VECTOR(9 downto 0);
signal xval_inv:			STD_LOGIC_VECTOR(9 downto 0);
signal yval_inv:			STD_LOGIC_VECTOR(9 downto 0);
signal zval	:			STD_LOGIC_VECTOR(7 downto 0);
signal rgb	:			STD_LOGIC_VECTOR(2 downto 0);
signal soundval:			STD_LOGIC_VECTOR(7 downto 0);
signal soundout:			STD_LOGIC;
signal cpu_reset_h:		STD_LOGIC;

component vectrex is
  port(
		cpu_areset   : in    std_logic;
		cpu_ahold    : in    std_logic;
		cpu_anmi     : in    std_logic;
		cpu_clk		: in    std_logic; --12 MHz
		dbg    : out std_logic_vector(15 downto 0);
		analog_sound_out    : out std_logic_vector(7 downto 0);
		analog_x_out    : out std_logic_vector(9 downto 0);
		analog_y_out    : out std_logic_vector(9 downto 0);
		analog_z_out    : out std_logic_vector(7 downto 0);
		p1_x				 : in std_logic_vector(7 downto 0);
		p1_y				 : in std_logic_vector(7 downto 0);
		p1_buttons		 : in std_logic_vector(3 downto 0);
		p2_x				 : in std_logic_vector(7 downto 0);
		p2_y				 : in std_logic_vector(7 downto 0);
		p2_buttons		 : in std_logic_vector(3 downto 0)
	);
end component;

attribute CLKDV_DIVIDE : string;
attribute CLKDV_DIVIDE of CLKDLLA : label is "4.5";
begin
	IBUFG0 : IBUFG port map (I=>I_CLK,  O=>clk_ibuf);
	CLKDLLA : CLKDLL port map (
		CLKIN  => clk_ibuf,
		RST    => reset_h,
		CLKFB  => clk_dlla_op0,
		CLK0   => clk_dlla_op0,
		CLKDV  => clk_dlla_div,
		CLK2X  => clk_dlla_2x
	);
	BUFG0 : BUFG port map (I=> clk_dlla_op0,O => clk_104);
	BUFG1 : BUFG port map (I=> clk_dlla_div,O => clk_12);
	BUFG2 : BUFG port map (I=> clk_dlla_2x,O => clk_pwm);

	pwmxh: pwmgen port map (
		inval => xval_pwm(9 downto 5),
		output => O_PWM_XH,
		clk => clk_pwm,
		reset => reset_h
	);
	pwmxl: pwmgen port map (
		inval => xval_pwm(4 downto 0),
		output => O_PWM_XL,
		clk => clk_pwm,
		reset => reset_h
	);
	pwmyh: pwmgen port map (
		inval => yval_pwm(9 downto 5),
		output => O_PWM_YH,
		clk => clk_pwm,
		reset => reset_h
	);
	pwmyl: pwmgen port map (
		inval => yval_pwm(4 downto 0),
		output => O_PWM_YL,
		clk => clk_pwm,
		reset => reset_h
	);
	
	audioda: deltasigma generic map (
			width => 8
		) port map (
			inval => soundval,
			output => soundout,
			clk => clk_pwm,
			reset => cpu_reset_h
	);

	myvectrex: vectrex port map (
		cpu_areset => cpu_reset_h,
		cpu_ahold => '0',
		cpu_anmi => '0',
		cpu_clk => clk_12,
		analog_sound_out => soundval,
		analog_x_out => xval,
		analog_y_out => yval,
		analog_z_out => zval,
		p1_x => player1_x,
		p1_y => player1_y,
		p1_buttons => player1_buttons,
		p2_x => player2_x,
		p2_y => player2_y,
		p2_buttons => player2_buttons,
		dbg => O_DBG
	);

	psxclk: process(clk_12, reset_h)
	begin
		if (reset_h='1') then
			clkc_psx<="0000";
		elsif rising_edge(clk_12) then
			clkc_psx<=clkc_psx+"0001";
			if (clkc_psx="0000") then
				clke_psx<='1';
			else
				clke_psx<='0';
			end if;
		end if;
	end process;

	psxcontroller : entity work.psxcontroller
		port map (
			psxdata => I_PSXDATA,
			psxcmd => O_PSXCMD,
			psxatt => O_PSXATT,
			psxclk => O_PSXCLK,
			psxack => I_PSXACK,
			buttons => psxbuttons,
			clk => clk_12,
			clken => clke_psx,
			reset_l => reset_l
		);

	--psxbuttons(15 downto 0): 
	--15:SQUARE 14:CROSS 13:CIRCLE 12:TRIANGLE 11:R1 10:L1 9:R2 8:L2 
	--7:LEFT 6:DOWN 5:RIGHT 4:UP 3:START 2:X 1:X 0:SELECT
	player2_x<="01111111";
	player2_y<="01111111";
	player2_buttons<="1111";
	player1_buttons(0)<=not psxbuttons(15);
	player1_buttons(1)<=not psxbuttons(14);
	player1_buttons(2)<=not psxbuttons(13);
	player1_buttons(3)<=not psxbuttons(12);
	player1_x<="11111111" when psxbuttons(7)='1' else
				"00000000" when psxbuttons(5)='1' else
				"10000000";
	player1_y<="11111111" when psxbuttons(6)='1' else
				"00000000" when psxbuttons(4)='1' else
				"10000000";
	
	cpu_reset_h<= psxbuttons(0);
	reset_h<=I_RESET;
	reset_l<=not I_RESET;
	xval_inv<=xval xor "1000000000";
	yval_inv<=yval xor "1000000000";
	
	yval_pwm<=xval_inv; --board swaps x and y
	xval_pwm<=yval_inv;

	O_VIDEO_Z<=zval(7 downto 4);
--	O_DBG<="1111111111111111";

	O_AUDIO_L<=soundout;
	O_AUDIO_R<=soundout;

	O_RDIS<='0';
	O_GDIS<='0';
	O_BDIS<='0';

end Behavioral;
